@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 #include <stdlib.h>
 #include <string.h>
@@ -48,7 +48,7 @@ struct _ThresholdDialog
   GtkWidget   *high_threshold_text;
   Histogram   *histogram;
 
-  int          drawable_id;
+  GimpDrawable *drawable;
   ImageMap     image_map;
   int          color;
   int          low_threshold;
@@ -65,10 +65,12 @@ static void   threshold_motion         (Tool *, GdkEventMotion *, gpointer);
 static void   threshold_cursor_update  (Tool *, GdkEventMotion *, gpointer);
 static void   threshold_control        (Tool *, int, gpointer);
 
+
 static ThresholdDialog *  threshold_new_dialog                 (void);
 static void               threshold_preview                    (ThresholdDialog *);
 static void               threshold_ok_callback                (GtkWidget *, gpointer);
 static void               threshold_cancel_callback            (GtkWidget *, gpointer);
+static gint               threshold_delete_callback            (GtkWidget *, GdkEvent *, gpointer);
 static void               threshold_preview_update             (GtkWidget *, gpointer);
 static void               threshold_low_threshold_text_update  (GtkWidget *, gpointer);
 static void               threshold_high_threshold_text_update (GtkWidget *, gpointer);
@@ -147,6 +149,9 @@ threshold_histogram_info (PixelRegion     *srcPR,
   int w, h;
   int value;
 
+  mask = NULL;
+  m    = NULL;
+
   td = (ThresholdDialog *) user_data;
 
   h = srcPR->h;
@@ -220,6 +225,10 @@ threshold_button_press (Tool           *tool,
 			GdkEventButton *bevent,
 			gpointer        gdisp_ptr)
 {
+  GDisplay *gdisp;
+
+  gdisp = gdisp_ptr;
+  tool->drawable = gimage_active_drawable (gdisp->gimage);
 }
 
 static void
@@ -265,7 +274,9 @@ threshold_control (Tool     *tool,
     case HALT :
       if (threshold_dialog)
 	{
+	  active_tool->preserve = TRUE;
 	  image_map_abort (threshold_dialog->image_map);
+	  active_tool->preserve = FALSE;
 	  threshold_dialog->image_map = NULL;
 	  threshold_cancel_callback (NULL, (gpointer) threshold_dialog);
 	}
@@ -304,6 +315,7 @@ tools_new_threshold ()
   tool->arrow_keys_func = standard_arrow_keys_func;
   tool->cursor_update_func = threshold_cursor_update;
   tool->control_func = threshold_control;
+  tool->preserve = FALSE;
 
   return tool;
 }
@@ -317,7 +329,7 @@ tools_free_threshold (Tool *tool)
 
   /*  Close the color select dialog  */
   if (threshold_dialog)
-    threshold_ok_callback (NULL, (gpointer) threshold_dialog);
+    threshold_cancel_callback (NULL, (gpointer) threshold_dialog);
 
   g_free (thresh);
 }
@@ -331,7 +343,7 @@ threshold_initialize (void *gdisp_ptr)
 
   if (drawable_indexed (gimage_active_drawable (gdisp->gimage)))
     {
-      message_box ("Threshold does not operate on indexed drawables.", NULL, NULL);
+      g_message ("Threshold does not operate on indexed drawables.");
       return;
     }
 
@@ -342,11 +354,11 @@ threshold_initialize (void *gdisp_ptr)
     if (!GTK_WIDGET_VISIBLE (threshold_dialog->shell))
       gtk_widget_show (threshold_dialog->shell);
 
-  threshold_dialog->drawable_id = gimage_active_drawable (gdisp->gimage);
-  threshold_dialog->color = drawable_color (threshold_dialog->drawable_id);
-  threshold_dialog->image_map = image_map_create (gdisp_ptr, threshold_dialog->drawable_id);
+  threshold_dialog->drawable = gimage_active_drawable (gdisp->gimage);
+  threshold_dialog->color = drawable_color (threshold_dialog->drawable);
+  threshold_dialog->image_map = image_map_create (gdisp_ptr, threshold_dialog->drawable);
   histogram_update (threshold_dialog->histogram,
-		    threshold_dialog->drawable_id,
+		    threshold_dialog->drawable,
 		    threshold_histogram_info,
 		    (void *) threshold_dialog);
   histogram_range (threshold_dialog->histogram,
@@ -368,7 +380,7 @@ static ActionAreaItem action_items[] =
   { "Cancel", threshold_cancel_callback, NULL, NULL }
 };
 
-ThresholdDialog *
+static ThresholdDialog *
 threshold_new_dialog ()
 {
   ThresholdDialog *td;
@@ -385,7 +397,14 @@ threshold_new_dialog ()
 
   /*  The shell and main vbox  */
   td->shell = gtk_dialog_new ();
+  gtk_window_set_wmclass (GTK_WINDOW (td->shell), "threshold", "Gimp");
   gtk_window_set_title (GTK_WINDOW (td->shell), "Threshold");
+
+  /* handle the wm close signal */
+  gtk_signal_connect (GTK_OBJECT (td->shell), "delete_event",
+		      GTK_SIGNAL_FUNC (threshold_delete_callback),
+		      td);
+
   vbox = gtk_vbox_new (FALSE, 2);
   gtk_container_border_width (GTK_CONTAINER (vbox), 2);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (td->shell)->vbox), vbox, TRUE, TRUE, 0);
@@ -469,7 +488,7 @@ static void
 threshold_preview (ThresholdDialog *td)
 {
   if (!td->image_map)
-    g_warning ("No image map");
+    g_message ("threshold_preview(): No image map");
   image_map_apply (td->image_map, threshold, (void *) td);
 }
 
@@ -484,15 +503,28 @@ threshold_ok_callback (GtkWidget *widget,
   if (GTK_WIDGET_VISIBLE (td->shell))
     gtk_widget_hide (td->shell);
 
+  active_tool->preserve = TRUE;
+
   if (!td->preview)
     image_map_apply (td->image_map, threshold, (void *) td);
-
   if (td->image_map)
     image_map_commit (td->image_map);
 
+  active_tool->preserve = FALSE;
+
   td->image_map = NULL;
 }
 
+static gint
+threshold_delete_callback (GtkWidget *w,
+			   GdkEvent *e,
+			   gpointer client_data) 
+{
+  threshold_cancel_callback (w, client_data);
+
+  return TRUE;
+}
+
 static void
 threshold_cancel_callback (GtkWidget *widget,
 			   gpointer   client_data)
@@ -505,7 +537,9 @@ threshold_cancel_callback (GtkWidget *widget,
 
   if (td->image_map)
     {
+      active_tool->preserve = TRUE;
       image_map_abort (td->image_map);
+      active_tool->preserve = FALSE;
       gdisplays_flush ();
     }
 
@@ -628,13 +662,17 @@ threshold_invoker (args)
   int success = TRUE;
   ThresholdDialog td;
   GImage *gimage;
-  int drawable_id;
+  GimpDrawable *drawable;
   int low_threshold;
   int high_threshold;
   int int_value;
   int x1, y1, x2, y2;
   void *pr;
 
+  drawable    = NULL;
+  low_threshold  = 0;
+  high_threshold = 0;
+
   /*  the gimage  */
   if (success)
     {
@@ -646,11 +684,14 @@ threshold_invoker (args)
   if (success)
     {
       int_value = args[1].value.pdb_int;
-      if (gimage == drawable_gimage (int_value))
-	drawable_id = int_value;
-      else
+      drawable = drawable_get_ID (int_value);
+      if (drawable == NULL || gimage != drawable_gimage (drawable))
 	success = FALSE;
     }
+  /*  make sure the drawable is not indexed color  */
+  if (success)
+    success = ! drawable_indexed (drawable);
+  
   /*  low threhsold  */
   if (success)
     {
@@ -675,21 +716,21 @@ threshold_invoker (args)
   /*  arrange to modify the levels  */
   if (success)
     {
-      td.color = drawable_color (drawable_id);
+      td.color = drawable_color (drawable);
       td.low_threshold = low_threshold;
       td.high_threshold = high_threshold;
 
       /*  The application should occur only within selection bounds  */
-      drawable_mask_bounds (drawable_id, &x1, &y1, &x2, &y2);
+      drawable_mask_bounds (drawable, &x1, &y1, &x2, &y2);
 
-      pixel_region_init (&srcPR, drawable_data (drawable_id), x1, y1, (x2 - x1), (y2 - y1), FALSE);
-      pixel_region_init (&destPR, drawable_shadow (drawable_id), x1, y1, (x2 - x1), (y2 - y1), TRUE);
+      pixel_region_init (&srcPR, drawable_data (drawable), x1, y1, (x2 - x1), (y2 - y1), FALSE);
+      pixel_region_init (&destPR, drawable_shadow (drawable), x1, y1, (x2 - x1), (y2 - y1), TRUE);
 
       for (pr = pixel_regions_register (2, &srcPR, &destPR); pr != NULL; pr = pixel_regions_process (pr))
 	threshold (&srcPR, &destPR, (void *) &td);
 
-      drawable_merge_shadow (drawable_id, TRUE);
-      drawable_update (drawable_id, x1, y1, (x2 - x1), (y2 - y1));
+      drawable_merge_shadow (drawable, TRUE);
+      drawable_update (drawable, x1, y1, (x2 - x1), (y2 - y1));
     }
 
   return procedural_db_return_args (&threshold_proc, success);
