@@ -21,6 +21,7 @@
 
 
 #include <glib.h>
+#include <libgimp/gimpfeatures.h>
 #include <libgimp/gimpenums.h>
 
 
@@ -29,6 +30,11 @@ extern "C" {
 #endif /* __cplusplus */
 
 
+extern const guint gimp_major_version;
+extern const guint gimp_minor_version;
+extern const guint gimp_micro_version;
+
+
 typedef struct _GPlugInInfo  GPlugInInfo;
 typedef struct _GTile        GTile;
 typedef struct _GDrawable    GDrawable;
@@ -359,7 +365,7 @@ void       gimp_image_clean_all             (gint32     image_ID);
 void       gimp_image_disable_undo          (gint32     image_ID);
 void       gimp_image_enable_undo           (gint32     image_ID);
 void       gimp_image_clean_all             (gint32     image_ID);
-void       gimp_image_flatten               (gint32     image_ID);
+gint32     gimp_image_flatten               (gint32     image_ID);
 void       gimp_image_lower_channel         (gint32     image_ID,
 					     gint32     channel_ID);
 void       gimp_image_lower_layer           (gint32     image_ID,
