@@ -49,8 +49,10 @@ static char ident[] = "@(#) GIMP PostScript/PDF file-plugin v1.04  20-Dec-97";
 #include <time.h>
 #include "gtk/gtk.h"
 #include "libgimp/gimp.h"
-#include "app/gimage.h"
-#include "app/gdisplay.h"
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #define STR_LENGTH 64
 
